@@ -4,17 +4,14 @@ This works as part of the lane/flowcell process step of the pipeline.
 """
 from collections import namedtuple
 import os
-import sys
-import glob
 
 from Bio.SeqIO.QualityIO import FastqGeneralIterator
 
-from bcbio import bam, broad, utils
+from bcbio import bam, utils
 from bcbio.bam import cram
 from bcbio.distributed.transaction import file_transaction
 from bcbio.ngsalign import (bowtie, bwa, tophat, bowtie2, mosaik,
                             novoalign, star)
-from bcbio.log import logger
 
 # Define a next-generation sequencing tool to plugin:
 # align_fn -- runs an aligner and generates SAM output
@@ -57,11 +54,13 @@ def align_to_sort_bam(fastq1, fastq2, aligner, data):
         align_dir_parts.append(data["disambiguate"]["genome_build"])
     align_dir = utils.safe_makedir(apply(os.path.join, align_dir_parts))
     if fastq1.endswith(".bam"):
-        out_bam = _align_from_bam(fastq1, aligner, data["align_ref"], data["sam_ref"],
+        out_bam = _align_from_bam(fastq1, aligner, utils.get_in(data, ("reference", aligner, "base")),
+                                  utils.get_in(data, ("reference", "fasta", "base")),
                                   names, align_dir, data)
         data["work_bam"] = out_bam
     else:
-        data = _align_from_fastq(fastq1, fastq2, aligner, data["align_ref"], data["sam_ref"],
+        data = _align_from_fastq(fastq1, fastq2, aligner, utils.get_in(data, ("reference", aligner, "base")),
+                                 utils.get_in(data, ("reference", "fasta", "base")),
                                  names, align_dir, data)
     if data["work_bam"] and utils.file_exists(data["work_bam"]):
         bam.index(data["work_bam"], data["config"])
@@ -95,13 +94,15 @@ def _align_from_fastq(fastq1, fastq2, aligner, align_ref, sam_ref, names,
     config = data["config"]
     align_fn = TOOLS[aligner].align_fn
     out = align_fn(fastq1, fastq2, align_ref, names, align_dir, data)
-    if isinstance(out, basestring) and out.endswith(".sam"):
-        if fastq2 is None and aligner in ["bwa", "bowtie2", "tophat2"]:
-            fastq1 = _remove_read_number(fastq1, out)
-        data["work_bam"] = sam_to_sort_bam(out, sam_ref, fastq1, fastq2, names, config)
-        return data
-    else:
+    # handle align functions that update the main data dictionary in place
+    if isinstance(out, dict):
+        assert "work_bam" in out
         return out
+    # handle output of raw SAM files that need to be converted to BAM
+    else:
+        work_bam = bam.sam_to_bam(out, config)
+        data["work_bam"] = bam.sort(work_bam, config)
+        return data
 
 def _remove_read_number(in_file, sam_file):
     """Work around problem with MergeBamAlignment with BWA and single end reads.
@@ -131,27 +132,3 @@ def _remove_read_number(in_file, sam_file):
                             name = name.rsplit("/", 1)[0]
                             out_handle.write("@%s\n%s\n+\n%s\n" % (name, seq, qual))
     return out_file
-
-def sam_to_sort_bam(sam_file, ref_file, fastq1, fastq2, names, config):
-    """Convert SAM file to merged and sorted BAM file.
-    """
-    picard = broad.runner_from_config(config)
-    base_dir = os.path.dirname(sam_file)
-
-    picard.run_fn("picard_index_ref", ref_file)
-    out_fastq_bam = picard.run_fn("picard_fastq_to_bam", fastq1, fastq2, base_dir, names)
-    out_bam = picard.run_fn("picard_sam_to_bam", sam_file, out_fastq_bam, ref_file,
-                            fastq2 is not None)
-    sort_bam = picard.run_fn("picard_sort", out_bam)
-
-    utils.save_diskspace(sam_file, "SAM converted to BAM", config)
-    utils.save_diskspace(out_fastq_bam, "Combined into output BAM %s" % out_bam, config)
-    utils.save_diskspace(out_bam, "Sorted to %s" % sort_bam, config)
-    # merge FASTQ files, only if barcoded samples in the work directory
-    if (os.path.commonprefix([fastq1, sort_bam]) ==
-             os.path.split(os.path.dirname(sort_bam))[0]
-          and not config["algorithm"].get("upload_fastq", True)):
-        utils.save_diskspace(fastq1, "Merged into output BAM %s" % out_bam, config)
-        if fastq2:
-            utils.save_diskspace(fastq2, "Merged into output BAM %s" % out_bam, config)
-    return sort_bam
