@@ -53,7 +53,8 @@ def merge_bam_files(bam_files, work_dir, config, out_file=None, batch=None):
             samtools = config_utils.get_program("samtools", config)
             resources = config_utils.get_resources("samtools", config)
             num_cores = config["algorithm"].get("num_cores", 1)
-            max_mem = resources.get("memory", "1G")
+            max_mem = config_utils.adjust_memory(resources.get("memory", "1G"),
+                                                 2, "decrease")
             batch_size = system.open_file_limit() - 100
             if len(bam_files) > batch_size:
                 bam_files = [merge_bam_files(xs, work_dir, config, out_file, i)
@@ -62,10 +63,9 @@ def merge_bam_files(bam_files, work_dir, config, out_file=None, batch=None):
                 with utils.chdir(tmpdir):
                     merge_cl = _bamtools_merge(bam_files)
                     with file_transaction(out_file) as tx_out_file:
-                        tx_out_prefix = os.path.splitext(tx_out_file)[0]
-                        with utils.tmpfile(dir=work_dir, prefix="bammergelist") as bam_file_list:
-                            bam_file_list = "%s.list" % os.path.splitext(out_file)[0]
-                            with open(bam_file_list, "w") as out_handle:
+                        with file_transaction("%s.list" % os.path.splitext(out_file)[0]) as tx_bam_file_list:
+                            tx_out_prefix = os.path.splitext(tx_out_file)[0]
+                            with open(tx_bam_file_list, "w") as out_handle:
                                 for f in sorted(bam_files):
                                     out_handle.write("%s\n" % f)
                             cmd = (merge_cl + " | "
@@ -96,4 +96,4 @@ def _bamtools_merge(bam_files):
                       "https://bcbio-nextgen.readthedocs.org/en/latest/contents/"
                       "parallel.html#tuning-systems-for-scale"
                       % (len(bam_files), system.open_file_limit()))
-    return "{bamtools} merge -list {bam_file_list}"
+    return "{bamtools} merge -list {tx_bam_file_list}"
