@@ -85,8 +85,13 @@ def delayed_bamprep_merge(samples, run_parallel):
 # ## Variant calling
 
 def _split_by_ready_regions(output_ext, file_key, dir_ext_fn):
-    """Organize splits into pre-built files generated by parallel_prep_region
+    """Organize splits into pre-built files generated by parallel_prep_region.
+
+    Handles special cases where we need to pass along samples that drive
+    batch processing -- like tumor samples in tumor/normal. This enables attaching the
+    normal files to these during grouping.
     """
+    batch_drivers = ["tumor"]
     def _do_work(data):
         if "region" in data and not data["region"][0] in ["nochrom", "noanalysis"]:
             bam_file = data[file_key]
@@ -97,7 +102,7 @@ def _split_by_ready_regions(output_ext, file_key, dir_ext_fn):
             out_dir = os.path.join(data["dirs"]["work"], dir_ext_fn(data))
             out_file = os.path.join(out_dir, "{noregion_base}{ext}".format(**locals()))
             out_parts = []
-            if not utils.file_exists(out_file):
+            if not utils.file_exists(out_file) or utils.get_in(data, ("metadata", "phenotype")) in batch_drivers:
                 out_region_dir = os.path.join(out_dir, chrom)
                 out_region_file = os.path.join(out_region_dir, "{base}{ext}".format(**locals()))
                 out_parts = [(data["region"], out_region_file)]
@@ -123,7 +128,7 @@ def parallel_variantcall_region(samples, run_parallel):
                 to_group.append(x[0])
             else:
                 extras.append(x)
-    split_fn = _split_by_ready_regions("-variants.vcf", "work_bam", genotype.get_variantcaller)
+    split_fn = _split_by_ready_regions("-variants.vcf.gz", "work_bam", genotype.get_variantcaller)
     if len(to_group) > 0:
         extras += group_combine_parts(to_group)
     return extras + grouped_parallel_split_combine(to_process, split_fn,
