@@ -6,6 +6,7 @@ next gen LIMS system or an on-file YAML configuration.
 import copy
 import itertools
 import os
+from contextlib import closing
 
 import yaml
 
@@ -16,6 +17,7 @@ from bcbio.pipeline import alignment, config_utils, genome
 from bcbio.solexa.flowcell import get_flowcell_info, get_fastq_dir
 from bcbio.variation import effects, genotype, population
 from bcbio.variation.cortex import get_sample_name
+from bcbio.bam.fastq import open_fastq
 
 def organize(dirs, config, run_info_yaml):
     """Organize run information from a passed YAML file or the Galaxy API.
@@ -115,6 +117,23 @@ def _check_for_duplicates(xs, attr, check_fn=None):
                          "Required to be unique for a project: %s\n"
                          "Problem found in these samples: %s" % (attr, dups, descrs))
 
+def _check_for_batch_clashes(xs):
+    """Check that batch names do not overlap with sample names.
+    """
+    names = set([x["description"] for x in xs])
+    dups = set([])
+    for x in xs:
+        batches = utils.get_in(x, ("metadata", "batch"))
+        if batches:
+            if not isinstance(batches, (list, tuple)):
+                batches = [batches]
+            for batch in batches:
+                if batch in names:
+                    dups.add(batch)
+    if len(dups) > 0:
+        raise ValueError("Batch names must be unique from sample descriptions.\n"
+                         "Clashing batch names: %s" % sorted(list(dups)))
+
 def _check_for_misplaced(xs, subkey, other_keys):
     """Ensure configuration keys are not incorrectly nested under other keys.
     """
@@ -135,14 +154,17 @@ ALGORITHM_KEYS = set(["platform", "aligner", "bam_clean", "bam_sort",
                       "align_split_size", "quality_bin",
                       "quality_format", "write_summary",
                       "merge_bamprep", "coverage", "coverage_bigwig",
-                      "coverage_depth", "coverage_interval", "ploidy",
+                      "coverage_interval", "ploidy",
                       "variantcaller", "variant_regions",
                       "mark_duplicates", "svcaller", "recalibrate",
                       "realign", "phasing", "validate",
                       "validate_regions", "validate_genome_build",
                       "clinical_reporting", "nomap_split_size",
                       "nomap_split_targets", "ensemble", "background",
-                      "disambiguate", "strandedness", "fusion_mode", "min_read_length"])
+                      "disambiguate", "strandedness", "fusion_mode", "min_read_length",
+                      "coverage_depth_min", "coverage_depth_max"] +
+                     # back compatibility
+                      ["coverage_depth"])
 
 def _check_algorithm_keys(item):
     """Check for unexpected keys in the algorithm section.
@@ -157,6 +179,64 @@ def _check_algorithm_keys(item):
                          "See configuration documentation for supported options:\n%s\n"
                          % (problem_keys, url))
 
+
+def _detect_fastq_format(in_file, MAX_RECORDS=1000):
+    ranges = {"sanger": (33, 73),
+              "solexa": (59, 104),
+              "illumina_1.3+": (64, 104),
+              "illumina_1.5+": (66, 104),
+              "illumina_1.8+": (35, 74)}
+
+    gmin, gmax = 99, 0
+    possible = set(ranges.keys())
+
+    with closing(open_fastq(in_file)) as in_handle:
+        four = itertools.islice(in_handle, 3, None, 4)
+        count = 0
+        for line in four:
+            if len(possible) == 1:
+                return possible
+            if count > MAX_RECORDS:
+                break
+            count += 1
+            vals = [ord(c) for c in line.rstrip()]
+            lmin = min(vals)
+            lmax = max(vals)
+            for encoding, (emin, emax) in ranges.items():
+                if encoding in possible:
+                    if lmin < emin or lmax > emax:
+                        possible.remove(encoding)
+
+    return possible
+
+def _check_quality_format(items):
+    """
+    Check if quality_format="standard" and fastq_format is not sanger
+    """
+    SAMPLE_FORMAT = {"illumina_1.3+": "illumina",
+                     "illumina_1.5+": "illumina",
+                     "illumina_1.8+": "standard",
+                     "solexa": "solexa",
+                     "sanger": "standard"}
+    fastq_extensions = ["fq.gz", "fastq.gz", ".fastq" ".fq"]
+
+    for item in items:
+        specified_format = item["algorithm"].get("quality_format", "").lower()
+        fastq_file = next((file for file in item.get('files', []) if
+                           any([ext for ext in fastq_extensions if ext in file])), None)
+
+        if fastq_file and specified_format:
+            fastq_format = _detect_fastq_format(fastq_file)
+            detected_encodings = set([SAMPLE_FORMAT[x] for x in fastq_format])
+            if detected_encodings:
+                if specified_format not in detected_encodings:
+                    raise ValueError("Quality format specified in the YAML "
+                                     "file might be a different encoding. "
+                                     "'%s' was specified but possible formats "
+                                     "detected were %s." % (specified_format,
+                                                            ", ".join(detected_encodings)))
+
+
 def _check_aligner(item):
     """Ensure specified aligner is valid choice.
     """
@@ -182,15 +262,19 @@ def _check_sample_config(items, in_file):
     """Identify common problems in input sample configuration files.
     """
     logger.info("Checking sample YAML configuration: %s" % in_file)
+    _check_quality_format(items)
     _check_for_duplicates(items, "lane")
     _check_for_duplicates(items, "description")
+    _check_for_batch_clashes(items)
     _check_for_misplaced(items, "algorithm",
                          ["resources", "metadata", "analysis",
                           "description", "genome_build", "lane", "files"])
+
     [_check_algorithm_keys(x) for x in items]
     [_check_aligner(x) for x in items]
     [_check_variantcaller(x) for x in items]
 
+
 # ## Read bcbio_sample.yaml files
 
 def _normalize_files(item, fc_dir):
@@ -249,6 +333,7 @@ def _run_info_from_yaml(fc_dir, run_info_yaml, config):
             fc_date = str(loaded["fc_date"]).replace(" ", "_")
         global_vars = global_config.pop("globals", {})
         loaded = loaded["details"]
+
     run_details = []
     for i, item in enumerate(loaded):
         item = _normalize_files(item, fc_dir)
@@ -274,6 +359,7 @@ def _run_info_from_yaml(fc_dir, run_info_yaml, config):
         item["algorithm"] = genome.abs_file_paths(item["algorithm"],
                                                   ignore_keys=["variantcaller", "realign", "recalibrate",
                                                                "phasing", "svcaller"])
+
         item["rgnames"] = prep_rg_names(item, config, fc_name, fc_date)
         item["test_run"] = global_config.get("test_run", False)
         run_details.append(item)
