@@ -99,24 +99,27 @@ def _do_run(cmd, checks, log_stdout=False):
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT, close_fds=True)
     debug_stdout = collections.deque(maxlen=100)
-    with contextlib.closing(s.stdout) as stdout:
-        while 1:
-            line = stdout.readline()
-            if line:
-                debug_stdout.append(line)
-                if log_stdout:
-                    logger_stdout.debug(line.rstrip())
-                else:
-                    logger.debug(line.rstrip())
-            exitcode = s.poll()
-            if exitcode is not None:
-                if exitcode is not None and exitcode != 0:
-                    error_msg = " ".join(cmd) if not isinstance(cmd, basestring) else cmd
-                    error_msg += "\n"
-                    error_msg += "".join(debug_stdout)
-                    raise subprocess.CalledProcessError(exitcode, error_msg)
-                else:
-                    break
+    while 1:
+        line = s.stdout.readline()
+        if line:
+            debug_stdout.append(line)
+            if log_stdout:
+                logger_stdout.debug(line.rstrip())
+            else:
+                logger.debug(line.rstrip())
+        exitcode = s.poll()
+        if exitcode is not None:
+            if exitcode is not None and exitcode != 0:
+                error_msg = " ".join(cmd) if not isinstance(cmd, basestring) else cmd
+                error_msg += "\n"
+                error_msg += "".join(debug_stdout)
+                s.communicate()
+                s.stdout.close()
+                raise subprocess.CalledProcessError(exitcode, error_msg)
+            else:
+                break
+    s.communicate()
+    s.stdout.close()
     # Check for problems not identified by shell return codes
     if checks:
         for check in checks:
