@@ -32,7 +32,8 @@ _cl_progs = [{"cmd": "bamtofastq", "name": "biobambam",
              {"cmd": "sambamba", "stdout_flag": "sambamba"},
              {"cmd": "qualimap", "args": "-h", "stdout_flag": "QualiMap"},
              {"cmd": "tophat", "args": "--version", "stdout_flag": "TopHat"},
-             {"cmd": "vcflib", "has_cl_version": False}]
+             {"cmd": "vcflib", "has_cl_version": False},
+             {"cmd": "featurecounts", "args": "-v", "stdout_flag": "featureCounts"}]
 
 def _broad_versioner(type):
     def get_version(config):
@@ -42,6 +43,9 @@ def _broad_versioner(type):
             return runner.get_gatk_version()
         elif type == "picard":
             return runner.get_picard_version("ViewSam")
+        elif type == "mutect":
+            runner = broad.runner_from_config(config, "mutect")
+            return runner.get_mutect_version()
         else:
             raise NotImplementedError(type)
     return get_version
@@ -82,7 +86,7 @@ _alt_progs = [{"name": "bcbio_variation",
                "version_fn": jar_versioner("bcbio_variation", "bcbio.variation")},
               {"name": "gatk", "version_fn": _broad_versioner("gatk")},
               {"name": "mutect",
-               "version_fn": jar_versioner("mutect", "muTect")},
+               "version_fn": _broad_versioner("mutect")},
               {"name": "picard", "version_fn": _broad_versioner("picard")},
               {"name": "rnaseqc",
                "version_fn": jar_versioner("rnaseqc", "RNA-SeQC")},
@@ -91,7 +95,10 @@ _alt_progs = [{"name": "bcbio_variation",
               {"name": "varscan",
                "version_fn": jar_versioner("varscan", "VarScan")},
               {"name": "oncofuse",
-               "version_fn": jar_versioner("Oncofuse", "Oncofuse")}]
+               "version_fn": jar_versioner("Oncofuse", "Oncofuse")},
+              {"name": "alientrimmer",
+               "version_fn": jar_versioner("AlienTrimmer", "AlienTrimmer")}
+]
 
 def _parse_from_stdoutflag(stdout, x):
     for line in stdout:
@@ -189,12 +196,12 @@ def _get_versions(config=None):
 def _get_versions_manifest():
     """Retrieve versions from a pre-existing manifest of installed software.
     """
-    all_pkgs = ["htseq", "cn.mops", "vt", "platypus-variant"] + \
+    all_pkgs = ["htseq", "cn.mops", "vt", "platypus-variant", "gatk-framework"] + \
                [p.get("name", p["cmd"]) for p in _cl_progs] + [p["name"] for p in _alt_progs]
     manifest_dir = os.path.join(config_utils.get_base_installdir(), "manifest")
     if os.path.exists(manifest_dir):
         out = []
-        for plist in ["brew", "python", "r", "debian", "custom"]:
+        for plist in ["toolplus", "brew", "python", "r", "debian", "custom"]:
             pkg_file = os.path.join(manifest_dir, "%s-packages.yaml" % plist)
             if os.path.exists(pkg_file):
                 with open(pkg_file) as in_handle:
