@@ -5,8 +5,7 @@ http://bioinformatics.bc.edu/marthlab/FreeBayes
 import os
 import shutil
 
-from bcbio import bam
-from bcbio.utils import file_exists
+from bcbio import bam, utils
 from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import config_utils
 from bcbio.pipeline.shared import subset_variant_regions
@@ -22,20 +21,20 @@ def region_to_freebayes(region):
     else:
         return region
 
-def _freebayes_options_from_config(items, aconfig, out_file, region=None):
+def _freebayes_options_from_config(items, config, out_file, region=None):
     opts = []
     opts += ["--ploidy", str(ploidy.get_ploidy(items, region))]
 
-    variant_regions = aconfig.get("variant_regions", None)
+    variant_regions = utils.get_in(config, ("algorithm", "variant_regions"))
     target = subset_variant_regions(variant_regions, region, out_file)
     if target:
         if isinstance(target, basestring) and os.path.isfile(target):
             opts += ["--targets", target]
         else:
             opts += ["--region", region_to_freebayes(target)]
-    #background = aconfig.get("call_background", None)
-    #if background and os.path.exists(background):
-    #    opts += ["--variant-input", background]
+    resources = config_utils.get_resources("freebayes", config)
+    if resources.get("options"):
+        opts += resources["options"]
     return opts
 
 def run_freebayes(align_bams, items, ref_file, assoc_files, region=None,
@@ -61,8 +60,8 @@ def _run_freebayes_caller(align_bams, items, ref_file, assoc_files,
     """
     config = items[0]["config"]
     if out_file is None:
-        out_file = "%s-variants.vcf" % os.path.splitext(align_bams[0])[0]
-    if not file_exists(out_file):
+        out_file = "%s-variants.vcf.gz" % os.path.splitext(align_bams[0])[0]
+    if not utils.file_exists(out_file):
         with file_transaction(out_file) as tx_out_file:
             for align_bam in align_bams:
                 bam.index(align_bam, config)
@@ -71,12 +70,12 @@ def _run_freebayes_caller(align_bams, items, ref_file, assoc_files,
             vcfallelicprimitives = config_utils.get_program("vcfallelicprimitives", config)
             vcfstreamsort = config_utils.get_program("vcfstreamsort", config)
             input_bams = " ".join("-b %s" % x for x in align_bams)
-            opts = " ".join(_freebayes_options_from_config(items, config["algorithm"],
-                                                           out_file, region))
+            opts = " ".join(_freebayes_options_from_config(items, config, out_file, region))
+            compress_cmd = "| bgzip -c" if out_file.endswith("gz") else ""
             cmd = ("{freebayes} -f {ref_file} {input_bams} {opts} | "
-                   "{vcffilter} -f 'QUAL > 5' -s | {vcfallelicprimitives} | {vcfstreamsort} > {tx_out_file}")
+                   "{vcffilter} -f 'QUAL > 5' -s | {vcfallelicprimitives} | {vcfstreamsort} "
+                   "{compress_cmd} > {tx_out_file}")
             do.run(cmd.format(**locals()), "Genotyping with FreeBayes", {})
-        clean_vcf_output(out_file, _clean_freebayes_output, "nodups")
     ann_file = annotation.annotate_nongatk_vcf(out_file, align_bams,
                                                assoc_files["dbsnp"],
                                                ref_file, config)
@@ -88,32 +87,35 @@ def _run_freebayes_paired(align_bams, items, ref_file, assoc_files,
 
     This is used for paired tumor / normal samples.
     """
+    config = items[0]["config"]
     if out_file is None:
-        out_file = "%s-paired-variants.vcf" % os.path.splitext(align_bams[0])[0]
-    if not file_exists(out_file):
+        out_file = "%s-paired-variants.vcf.gz" % os.path.splitext(align_bams[0])[0]
+    if not utils.file_exists(out_file):
         with file_transaction(out_file) as tx_out_file:
             paired = get_paired_bams(align_bams, items)
             if not paired.normal_bam:
                 raise ValueError("Require both tumor and normal BAM files for FreeBayes cancer calling")
-            config = items[0]["config"]
+
             vcfsamplediff = config_utils.get_program("vcfsamplediff", config)
+            vcffilter = config_utils.get_program("vcffilter", config)
             freebayes = config_utils.get_program("freebayes", config)
-            opts = " ".join(
-                _freebayes_options_from_config(items, config["algorithm"],
-                                               out_file, region))
+            opts = " ".join(_freebayes_options_from_config(items, config, out_file, region))
             opts += " -f {}".format(ref_file)
             # NOTE: The first sample name in the vcfsamplediff call is
             # the one supposed to be the *germline* one
-            cl = ("{freebayes} --pooled-discrete --pvar 0.7"
-                  " --genotype-qualities {opts} {paired.tumor_bam}"
-                  " {paired.normal_bam} | {vcfsamplediff} -s VT"
-                  " {paired.normal_name} {paired.tumor_name}"
-                  " - >  {tx_out_file}")
+
+            # NOTE: -s in vcfsamplediff (strict checking: i.e., require no
+            # reads in the germline to call somatic) is not used as it is
+            # too stringent
+            compress_cmd = "| bgzip -c" if out_file.endswith("gz") else ""
+            cl = ("{freebayes} --pooled-discrete --genotype-qualities "
+                  "{opts} {paired.tumor_bam} {paired.normal_bam} "
+                  "| {vcffilter} -f 'QUAL > 1' -s "
+                  "| {vcfsamplediff} VT {paired.normal_name} {paired.tumor_name} - "
+                  "{compress_cmd} >  {tx_out_file}")
             bam.index(paired.tumor_bam, config)
             bam.index(paired.normal_bam, config)
             do.run(cl.format(**locals()), "Genotyping paired variants with FreeBayes", {})
-        clean_vcf_output(out_file, _clean_freebayes_output, "nodups")
-
     ann_file = annotation.annotate_nongatk_vcf(out_file, align_bams,
                                                assoc_files["dbsnp"], ref_file,
                                                config)
@@ -129,6 +131,10 @@ def _move_vcf(orig_file, new_file):
 
 def _clean_freebayes_output(line):
     """Clean FreeBayes output to make post-processing with GATK happy.
+
+    XXX Not applied on recent versions which fix issues to be more compatible
+    with bgzip output, but retained in case of need.
+
     - Remove lines from FreeBayes outputs where REF/ALT are identical:
       2       22816178        .       G       G       0.0339196
       or there are multiple duplicate alleles:
@@ -150,9 +156,9 @@ def clean_vcf_output(orig_file, clean_fn, name="clean"):
     """Provide framework to clean a file in-place, with the specified clean
     function.
     """
-    base, ext = os.path.splitext(orig_file)
+    base, ext = utils.splitext_plus(orig_file)
     out_file = "{0}-{1}{2}".format(base, name, ext)
-    if not file_exists(out_file):
+    if not utils.file_exists(out_file):
         with open(orig_file) as in_handle:
             with file_transaction(out_file) as tx_out_file:
                 with open(tx_out_file, "w") as out_handle:
