@@ -1,13 +1,4 @@
-"""Provide SNP, indel calling and variation analysis using GATK genotyping tools.
-
-Genotyping:
-
-http://www.broadinstitute.org/gsa/wiki/index.php/Best_Practice_Variant_Detection_with_the_GATK_v3
-http://www.broadinstitute.org/gsa/wiki/index.php/Unified_genotyper
-
-Variant Evaluation:
-
-http://www.broadinstitute.org/gsa/wiki/index.php/VariantEval
+"""Provide SNP and indel calling using GATK genotyping tools.
 """
 import os
 import collections
@@ -34,15 +25,17 @@ def _shared_gatk_call_prep(align_bams, ref_file, config, dbsnp, region, out_file
     broad_runner.run_fn("picard_index_ref", ref_file)
     for x in align_bams:
         bam.index(x, config)
-    coverage_depth = config["algorithm"].get("coverage_depth", "high").lower()
+    # GATK can only downsample to a minimum of 200
+    coverage_depth_max = max(200, utils.get_in(config, ("algorithm", "coverage_depth_max"), 10000))
+    coverage_depth_min = utils.get_in(config, ("algorithm", "coverage_depth_min"), 4)
     variant_regions = config["algorithm"].get("variant_regions", None)
-    confidence = "4.0" if coverage_depth in ["low"] else "30.0"
+    confidence = "4.0" if coverage_depth_min < 4 else "30.0"
     region = subset_variant_regions(variant_regions, region, out_file)
 
     params = ["-R", ref_file,
               "--standard_min_confidence_threshold_for_calling", confidence,
               "--standard_min_confidence_threshold_for_emitting", confidence,
-              "--downsample_to_coverage", "250",
+              "--downsample_to_coverage", str(coverage_depth_max),
               "--downsampling_type", "BY_SAMPLE",
               ]
     for a in annotation.get_gatk_annotations(config):
@@ -60,14 +53,15 @@ def unified_genotyper(align_bams, items, ref_file, assoc_files,
     """Perform SNP genotyping on the given alignment file.
     """
     if out_file is None:
-        out_file = "%s-variants.vcf" % os.path.splitext(align_bams[0])[0]
+        out_file = "%s-variants.vcf.gz" % os.path.splitext(align_bams[0])[0]
     if not file_exists(out_file):
+        config = items[0]["config"]
         broad_runner, params = \
             _shared_gatk_call_prep(align_bams, ref_file, items[0]["config"], assoc_files["dbsnp"],
                                    region, out_file)
         if (not isinstance(region, (list, tuple)) and
                 not all(has_aligned_reads(x, region) for x in align_bams)):
-            vcfutils.write_empty_vcf(out_file)
+            vcfutils.write_empty_vcf(out_file, config)
         else:
             with file_transaction(out_file) as tx_out_file:
                 params += ["-T", "UnifiedGenotyper",
@@ -85,15 +79,16 @@ def haplotype_caller(align_bams, items, ref_file, assoc_files,
     This requires the full non open-source version of GATK.
     """
     if out_file is None:
-        out_file = "%s-variants.vcf" % os.path.splitext(align_bams[0])[0]
+        out_file = "%s-variants.vcf.gz" % os.path.splitext(align_bams[0])[0]
     if not file_exists(out_file):
+        config = items[0]["config"]
         broad_runner, params = \
             _shared_gatk_call_prep(align_bams, ref_file, items[0]["config"], assoc_files["dbsnp"],
                                    region, out_file)
         assert broad_runner.gatk_type() == "restricted", \
             "Require full version of GATK 2.4+ for haplotype calling"
         if not all(has_aligned_reads(x, region) for x in align_bams):
-            vcfutils.write_empty_vcf(out_file)
+            vcfutils.write_empty_vcf(out_file, config)
         else:
             with file_transaction(out_file) as tx_out_file:
                 params += ["-T", "HaplotypeCaller",
@@ -154,14 +149,14 @@ def variant_filtration(call_file, ref_file, vrn_files, data):
         snp_filter_file = _variant_filtration_snp(snp_file, ref_file, vrn_files, data)
         indel_filter_file = _variant_filtration_indel(indel_file, ref_file, vrn_files, data)
         orig_files = [snp_filter_file, indel_filter_file]
-        out_file = "{base}combined.vcf".format(base=os.path.commonprefix(orig_files))
+        out_file = "{base}combined.vcf.gz".format(base=os.path.commonprefix(orig_files))
         return vcfutils.combine_variant_files(orig_files, out_file, ref_file, config)
 
 def _apply_variant_recal(broad_runner, snp_file, ref_file, recal_file,
                          tranch_file, filter_type):
     """Apply recalibration details, returning filtered VCF file.
     """
-    base, ext = os.path.splitext(snp_file)
+    base, ext = utils.splitext_plus(snp_file)
     out_file = "{base}-{filter}filter{ext}".format(base=base, ext=ext,
                                                    filter=filter_type)
     if not file_exists(out_file):
@@ -179,8 +174,8 @@ def _apply_variant_recal(broad_runner, snp_file, ref_file, recal_file,
 def _shared_variant_filtration(filter_type, snp_file, ref_file, vrn_files, variantcaller):
     """Share functionality for filtering variants.
     """
-    recal_file = "{base}.recal".format(base=os.path.splitext(snp_file)[0])
-    tranches_file = "{base}.tranches".format(base=os.path.splitext(snp_file)[0])
+    recal_file = "{base}.recal".format(base=utils.splitext_plus(snp_file)[0])
+    tranches_file = "{base}.tranches".format(base=utils.splitext_plus(snp_file)[0])
     params = ["-T", "VariantRecalibrator",
               "-R", ref_file,
               "--input", snp_file,
@@ -225,7 +220,7 @@ def _variant_filtration_snp(snp_file, ref_file, vrn_files, data):
         return vfilter.hard_w_expression(snp_file, " || ".join(filters), data, filter_type)
     else:
         # also check if we've failed recal and needed to do strict filtering
-        filter_file = "{base}-filter{ext}.vcf".format(base=os.path.splitext(snp_file)[0], ext=filter_type)
+        filter_file = "{base}-filter{ext}.vcf.gz".format(base=utils.splitext_plus(snp_file)[0], ext=filter_type)
         if file_exists(filter_file):
             config["algorithm"]["coverage_interval"] = "regional"
             return _variant_filtration_snp(snp_file, ref_file, vrn_files, data)
@@ -263,7 +258,7 @@ def _variant_filtration_indel(snp_file, ref_file, vrn_files, data):
         return vfilter.hard_w_expression(snp_file, filterexp, data, filter_type)
     else:
         # also check if we've failed recal and needed to do strict filtering
-        filter_file = "{base}-filter{ext}.vcf".format(base=os.path.splitext(snp_file)[0], ext=filter_type)
+        filter_file = "{base}-filter{ext}.vcf.gz".format(base=utils.splitext_plus(snp_file)[0], ext=filter_type)
         if file_exists(filter_file):
             config["algorithm"]["coverage_interval"] = "regional"
             return _variant_filtration_indel(snp_file, ref_file, vrn_files, data)
@@ -286,104 +281,6 @@ def _variant_filtration_indel(snp_file, ref_file, vrn_files, data):
         return _apply_variant_recal(broad_runner, snp_file, ref_file, recal_file,
                                     tranches_file, filter_type)
 
-# ## Variant evaluation
-
-def gatk_evaluate_variants(vcf_file, ref_file, config, dbsnp=None):
-    """Evaluate variants, return SNP counts and Transition/Transversion ratios.
-    """
-    runner = broad.runner_from_config(config)
-    eval_file = variant_eval(vcf_file, ref_file, dbsnp, runner)
-    stats = _extract_eval_stats(eval_file)
-    return _format_stats(stats['called'])
-
-def _format_stats(stats):
-    """Convert statistics into high level summary of major variables.
-    """
-    total = sum(itertools.chain.from_iterable(s.itervalues() for s in stats.itervalues()))
-    if total > 0:
-        dbsnp = sum(stats['known'].itervalues()) / float(total) * 100.0
-    else:
-        dbsnp = -1.0
-    tv_dbsnp = stats['known']['tv']
-    ti_dbsnp = stats['known']['ti']
-    tv_novel = stats['novel']['tv']
-    ti_novel = stats['novel']['ti']
-    if tv_novel > 0 and tv_dbsnp > 0:
-        titv_all = float(ti_novel + ti_dbsnp) / float(tv_novel + tv_dbsnp)
-        titv_dbsnp = float(ti_dbsnp) / float(tv_dbsnp)
-        titv_novel = float(ti_novel) / float(tv_novel)
-    else:
-        titv_all, titv_dbsnp, titv_novel = (-1.0, -1.0, -1.0)
-    return dict(total=total, dbsnp_pct=dbsnp, titv_all=titv_all,
-                titv_dbsnp=titv_dbsnp, titv_novel=titv_novel)
-
-def _extract_eval_stats(eval_file):
-    """Parse statistics of interest from GATK output file.
-    """
-    stats = dict()
-    for snp_type in ['called', 'filtered']:
-        stats[snp_type] = dict()
-        for dbsnp_type in ['known', 'novel']:
-            stats[snp_type][dbsnp_type] = dict(ti=0, tv=0)
-    for line in _eval_analysis_type(eval_file, "Ti/Tv Variant Evaluator"):
-        if line[1:3] == ['dbsnp', 'eval']:
-            snp_type = line[3]
-            dbsnp_type = line[5]
-            try:
-                cur = stats[snp_type][dbsnp_type]
-            except KeyError:
-                cur = None
-            if cur:
-                stats[snp_type][dbsnp_type]["ti"] = int(line[6])
-                stats[snp_type][dbsnp_type]["tv"] = int(line[7])
-    return stats
-
-def _eval_analysis_type(in_file, analysis_name):
-    """Retrieve data lines associated with a particular analysis.
-    """
-    supported_versions = ["v0.2", "v1.0", "v1.1"]
-    with open(in_file) as in_handle:
-        # read until we reach the analysis
-        for line in in_handle:
-            if line.startswith(("##:GATKReport", "#:GATKReport")):
-                version = line.split()[0].split(".", 1)[-1].split(":")[0]
-                assert version in supported_versions, \
-                       "Unexpected GATKReport version: {0}".format(version)
-                if line.find(analysis_name) > 0:
-                    break
-        # read off header lines
-        for _ in range(1):
-            in_handle.next()
-        # read the table until a blank line
-        for line in in_handle:
-            if not line.strip():
-                break
-            parts = line.rstrip("\n\r").split()
-            yield parts
-
-def variant_eval(vcf_in, ref_file, dbsnp, picard):
-    """Evaluate variants in comparison with dbSNP reference.
-    """
-    out_file = "%s.eval" % os.path.splitext(vcf_in)[0]
-    if not file_exists(out_file):
-        with file_transaction(out_file) as tx_out_file:
-            params = ["-T", "VariantEval",
-                      "-R", ref_file,
-                      "--eval", vcf_in,
-                      "-L", vcf_in,
-                      "--dbsnp", dbsnp,
-                      "-ST", "Filter",
-                      "-o", tx_out_file,
-                      "-l", "INFO",
-                      "--doNotUseAllStandardModules",
-                      "--evalModule", "CompOverlap",
-                      "--evalModule", "CountVariants",
-                      "--evalModule", "TiTvVariantEvaluator",
-                      "--evalModule", "ValidationReport",
-                      "--stratificationModule", "Filter"]
-            picard.run_gatk(params)
-    return out_file
-
 # ## High level functionality to run genotyping in parallel
 
 def get_variantcaller(data):
@@ -404,7 +301,7 @@ def combine_multiple_callers(data):
         final = grouped_calls[0]
         def orig_variantcaller_order(x):
             return final["config"]["algorithm"]["orig_variantcaller"].index(x["variantcaller"])
-        if len(ready_calls) > 1:
+        if len(ready_calls) > 1 and "orig_variantcaller" in final["config"]["algorithm"]:
             final["variants"] = sorted(ready_calls, key=orig_variantcaller_order)
         else:
             final["variants"] = ready_calls
@@ -441,7 +338,7 @@ def parallel_variantcall(sample_info, parallel_fn):
         else:
             finished.append(x)
     if len(to_process) > 0:
-        split_fn = process_bam_by_chromosome("-variants.vcf", "work_bam",
+        split_fn = process_bam_by_chromosome("-variants.vcf.gz", "work_bam",
                                              dir_ext_fn=get_variantcaller)
         processed = grouped_parallel_split_combine(
             to_process, split_fn, multi.group_batches, parallel_fn,
@@ -463,25 +360,26 @@ def get_variantcallers():
 def variantcall_sample(data, region=None, out_file=None):
     """Parallel entry point for doing genotyping of a region of a sample.
     """
-    safe_makedir(os.path.dirname(out_file))
-    sam_ref = data["sam_ref"]
-    config = data["config"]
-    caller_fns = get_variantcallers()
-    caller_fn = caller_fns[config["algorithm"].get("variantcaller", "gatk")]
-    if isinstance(data["work_bam"], basestring):
-        align_bams = [data["work_bam"]]
-        items = [data]
-    else:
-        align_bams = data["work_bam"]
-        items = data["work_items"]
-    call_file = "%s-raw%s" % os.path.splitext(out_file)
-    call_file = caller_fn(align_bams, items, sam_ref,
-                          data["genome_resources"]["variation"],
-                          region, call_file)
-    if data["config"]["algorithm"].get("phasing", False) == "gatk":
-        call_file = phasing.read_backed_phasing(call_file, align_bams, sam_ref, region, config)
-    utils.symlink_plus(call_file, out_file)
-    if "work_items" in data:
-        del data["work_items"]
+    if out_file is None or not os.path.exists(out_file) or not os.path.lexists(out_file):
+        safe_makedir(os.path.dirname(out_file))
+        sam_ref = data["sam_ref"]
+        config = data["config"]
+        caller_fns = get_variantcallers()
+        caller_fn = caller_fns[config["algorithm"].get("variantcaller", "gatk")]
+        if isinstance(data["work_bam"], basestring):
+            align_bams = [data["work_bam"]]
+            items = [data]
+        else:
+            align_bams = data["work_bam"]
+            items = data["work_items"]
+        call_file = "%s-raw%s" % utils.splitext_plus(out_file)
+        call_file = caller_fn(align_bams, items, sam_ref,
+                              data["genome_resources"]["variation"],
+                              region, call_file)
+        if data["config"]["algorithm"].get("phasing", False) == "gatk":
+            call_file = phasing.read_backed_phasing(call_file, align_bams, sam_ref, region, config)
+        utils.symlink_plus(call_file, out_file)
+        if "work_items" in data:
+            del data["work_items"]
     data["vrn_file"] = out_file
     return [data]
