@@ -15,17 +15,13 @@ from bcbio.pipeline import config_utils
 from bcbio.provenance import do, programs
 from bcbio.variation import vcfutils
 
-def prep_gemini_db(fnames, call_id, samples, data):
+def prep_gemini_db(fnames, call_info, samples, data):
     """Prepare a gemini database from VCF inputs prepared with snpEff.
     """
     out_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "gemini"))
-    gemini_db = os.path.join(out_dir, "-".join(call_id) + ".db")
-    is_population = len(fnames) > 1
-    if is_population:
-        name, caller = call_id
-        gemini_vcf = get_multisample_vcf(fnames, name, caller, data)
-    else:
-        gemini_vcf = fnames[0]
+    name, caller, is_batch = call_info
+    gemini_db = os.path.join(out_dir, "%s-%s.db" % (name, caller))
+    gemini_vcf = get_multisample_vcf(fnames, name, caller, data)
     use_gemini_quick = (do_db_build(samples, check_gemini=False) and
                         any(vcfutils.vcf_has_variants(f) for f in fnames))
     if not utils.file_exists(gemini_db) and use_gemini_quick:
@@ -38,24 +34,53 @@ def prep_gemini_db(fnames, call_id, samples, data):
                 else:
                     gemini_ver = None
                 # Recent versions of gemini allow loading only passing variants
+                load_opts = ""
                 if not gemini_ver or LooseVersion(gemini_ver) > LooseVersion("0.6.2.1"):
-                    load_opts = "--passonly"
-                else:
-                    load_opts = ""
+                    load_opts += " --passonly"
+                # For small test files, skip gene table loading which takes a long time
+                if gemini_ver and LooseVersion(gemini_ver) > LooseVersion("0.6.4"):
+                    if _is_small_vcf(gemini_vcf):
+                        load_opts += " --skip-gene-tables"
+                    if "/test_automated_output/" in gemini_vcf:
+                        load_opts += " --test-mode"
                 num_cores = data["config"]["algorithm"].get("num_cores", 1)
                 cmd = "{gemini} load {load_opts} -v {gemini_vcf} -t snpEff --cores {num_cores} {tx_gemini_db}"
                 cmd = cmd.format(**locals())
-                do.run(cmd, "Create gemini database for %s" % str(call_id), data)
-    return [[call_id, {"db": gemini_db if utils.file_exists(gemini_db) else None,
-                       "vcf": gemini_vcf if is_population else None}]]
+                do.run(cmd, "Create gemini database for %s %s" % (name, caller), data)
+    return [[(name, caller), {"db": gemini_db if utils.file_exists(gemini_db) else None,
+                              "vcf": gemini_vcf if is_batch else None}]]
+
+def _is_small_vcf(vcf_file):
+    """Check for small VCFs which we want to analyze quicker.
+    """
+    count = 0
+    small_thresh = 250
+    with utils.open_gzipsafe(vcf_file) as in_handle:
+        for line in in_handle:
+            if not line.startswith("#"):
+                count += 1
+            if count > small_thresh:
+                return False
+    return True
 
 def get_multisample_vcf(fnames, name, caller, data):
     """Retrieve a multiple sample VCF file in a standard location.
+
+    Handles inputs with multiple repeated input files from batches.
     """
+    unique_fnames = []
+    for f in fnames:
+        if f not in unique_fnames:
+            unique_fnames.append(f)
     out_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "gemini"))
-    gemini_vcf = os.path.join(out_dir, "%s-%s.vcf" % (name, caller))
-    return vcfutils.combine_variant_files(fnames, gemini_vcf, data["sam_ref"],
-                                          data["config"])
+    if len(unique_fnames) > 1:
+        gemini_vcf = os.path.join(out_dir, "%s-%s.vcf.gz" % (name, caller))
+        return vcfutils.merge_variant_files(unique_fnames, gemini_vcf, data["sam_ref"],
+                                            data["config"])
+    else:
+        gemini_vcf = os.path.join(out_dir, "%s-%s%s" % (name, caller, utils.splitext_plus(unique_fnames[0])[1]))
+        utils.symlink_plus(unique_fnames[0], gemini_vcf)
+        return gemini_vcf
 
 def _has_gemini(config):
     try:
@@ -105,15 +130,16 @@ def _group_by_batches(samples, check_fn):
     for data in [x[0] for x in samples]:
         if check_fn(data):
             batch = data.get("metadata", {}).get("batch")
+            name = str(data["name"][-1])
             if batch:
                 out_retrieve.append((str(batch), data))
             else:
-                out_retrieve.append((str(data["name"][-1]), data))
+                out_retrieve.append((name, data))
             for vrn in data["variants"]:
                 if batch:
                     batch_groups[(str(batch), vrn["variantcaller"])].append((vrn["vrn_file"], data))
                 else:
-                    singles.append((str(data["name"][-1]), vrn["variantcaller"], data, vrn["vrn_file"]))
+                    singles.append((name, vrn["variantcaller"], data, vrn["vrn_file"]))
         else:
             extras.append(data)
     return batch_groups, singles, out_retrieve, extras
@@ -129,10 +155,10 @@ def prep_db_parallel(samples, parallel_fn):
     has_batches = False
     for (name, caller), info in batch_groups.iteritems():
         fnames = [x[0] for x in info]
-        to_process.append([fnames, (str(name), caller), [x[1] for x in info], info[0][1]])
+        to_process.append([fnames, (str(name), caller, True), [x[1] for x in info], info[0][1]])
         has_batches = True
     for name, caller, data, fname in singles:
-        to_process.append([[fname], (str(name), caller), [data], data])
+        to_process.append([[fname], (str(name), caller, False), [data], data])
     if len(samples) > 0 and not do_db_build([x[0] for x in samples], check_gemini=False) and not has_batches:
         return samples
     output = parallel_fn("prep_gemini_db", to_process)
