@@ -7,11 +7,12 @@ from distutils.version import LooseVersion
 import os
 import shutil
 
+from bcbio import utils
 from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import config_utils
 from bcbio.provenance import do, programs
 from bcbio.utils import file_exists, append_stem
-from bcbio.variation import freebayes, samtools
+from bcbio.variation import freebayes, samtools, vcfutils
 from bcbio.variation.vcfutils import (combine_variant_files, write_empty_vcf,
                                       get_paired_bams, is_paired_analysis)
 
@@ -20,7 +21,6 @@ import pysam
 
 def run_varscan(align_bams, items, ref_file, assoc_files,
                 region=None, out_file=None):
-
     if is_paired_analysis(align_bams, items):
         call_file = samtools.shared_variantcall(_varscan_paired, "varscan",
                                                 align_bams, ref_file, items,
@@ -69,7 +69,9 @@ def _varscan_paired(align_bams, ref_file, items, target_regions, out_file):
         raise ValueError("Require both tumor and normal BAM files for VarScan cancer calling")
 
     if not file_exists(out_file):
-        base, ext = os.path.splitext(out_file)
+        orig_out_file = out_file
+        out_file = orig_out_file.replace(".vcf.gz", ".vcf")
+        base, ext = utils.splitext_plus(out_file)
         cleanup_files = []
         for fname, mpext in [(paired.normal_bam, "normal"), (paired.tumor_bam, "tumor")]:
             mpfile = "%s-%s.mpileup" % (base, mpext)
@@ -88,7 +90,7 @@ def _varscan_paired(align_bams, ref_file, items, target_regions, out_file):
         # just skip the rest of the analysis (VarScan will hang otherwise)
 
         if any(os.stat(filename).st_size == 0 for filename in cleanup_files):
-            write_empty_vcf(out_file)
+            write_empty_vcf(orig_out_file, config)
             return
 
         # First index is normal, second is tumor
@@ -127,7 +129,7 @@ def _varscan_paired(align_bams, ref_file, items, target_regions, out_file):
             _fix_varscan_vcf(indel_file, paired.normal_name, paired.tumor_name)
 
         if not to_combine:
-            write_empty_vcf(out_file)
+            write_empty_vcf(orig_out_file, config)
             return
 
         out_file = combine_variant_files([snp_file, indel_file],
@@ -137,11 +139,16 @@ def _varscan_paired(align_bams, ref_file, items, target_regions, out_file):
         # Remove cleanup files
 
         for extra_file in cleanup_files:
-            os.remove(extra_file)
+            for ext in ["", ".gz", ".gz.tbi"]:
+                if os.path.exists(extra_file + ext):
+                    os.remove(extra_file + ext)
 
         if os.path.getsize(out_file) == 0:
             write_empty_vcf(out_file)
 
+        if orig_out_file.endswith(".gz"):
+            vcfutils.bgzip_and_index(out_file, config)
+
 
 def _fix_varscan_vcf(orig_file, normal_name, tumor_name):
     """Fixes issues with the standard VarScan VCF output.
@@ -275,9 +282,11 @@ def _create_sample_list(in_bams, vcf_file):
 def _varscan_work(align_bams, ref_file, items, target_regions, out_file):
     """Perform SNP and indel genotyping with VarScan.
     """
-
     config = items[0]["config"]
 
+    orig_out_file = out_file
+    out_file = orig_out_file.replace(".vcf.gz", ".vcf")
+
     max_read_depth = "1000"
     version = programs.jar_versioner("varscan", "VarScan")(config)
     if version < "v2.3.6":
@@ -317,6 +326,9 @@ def _varscan_work(align_bams, ref_file, items, target_regions, out_file):
     else:
         freebayes.clean_vcf_output(out_file, _clean_varscan_line)
 
+    if orig_out_file.endswith(".gz"):
+        vcfutils.bgzip_and_index(out_file, config)
+
 def _clean_varscan_line(line):
     """Avoid lines with non-GATC bases, ambiguous output bases make GATK unhappy.
     """
