@@ -85,7 +85,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         """
         DlInfo = collections.namedtuple("DlInfo", "fname dirname version")
         download_data = [DlInfo("110106_FC70BUKAAXX.tar.gz", None, None),
-                         DlInfo("genomes_automated_test.tar.gz", "genomes", 13),
+                         DlInfo("genomes_automated_test.tar.gz", "genomes", 14),
                          DlInfo("110907_ERP000591.tar.gz", None, None),
                          DlInfo("100326_FC6107FAAXX.tar.gz", None, 5),
                          DlInfo("tcga_benchmark.tar.gz", None, 2)]
@@ -143,6 +143,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
             subprocess.check_call(cl)
 
     @attr(stranded=True)
+    @attr(rnaseq=True)
     def test_2_stranded(self):
         """Run an RNA-seq analysis with TopHat and generate gene-level counts.
         """
@@ -195,6 +196,8 @@ class AutomatedAnalysisTest(unittest.TestCase):
             subprocess.check_call(cl)
 
     @attr(explant=True)
+    @attr(singleend=True)
+    @attr(rnaseq=True)
     def test_explant(self):
         """
         Run an explant RNA-seq analysis with TopHat and generate gene-level counts.
@@ -260,6 +263,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
 
     @attr(speed=2)
     @attr(cancer=True)
+    @attr(cancermulti=True)
     def test_7_cancer(self):
         """Test paired tumor-normal calling using multiple calling approaches: MuTect, VarScan, FreeBayes.
         """
@@ -274,7 +278,6 @@ class AutomatedAnalysisTest(unittest.TestCase):
     @attr(cancer=True)
     def test_7_cancer_nonormal(self):
         """Test cancer calling without normal samples or with normal VCF panels.
-        XXX Not yet working
         """
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
